INTEGER          = 'INTEGER'  
FLOAT            = 'FLOAT'  
PLUS             = 'PLUS'  
MINUS            = 'MINUS'  
MUL              = 'MUL'  
DIV              = 'DIV'  
LPAREN           = 'LPAREN'  
RPAREN           = 'RPAREN'  
IDENTIFIER       = 'IDENTIFIER'  
ASSIGN           = 'ASSIGN'  
SEMI             = 'SEMI'      # Semicolon  
EOF              = 'EOF'       # End-of-file  
INPUT            = 'INPUT'     # van muji  
PRINT            = 'PRINT'     # her muji  
VAR_DECL         = 'VAR_DECL'  # dey muji  
IF               = 'IF'        # yedi muji  
ELIF             = 'ELIF'      # natra bhane muji  
ELSE             = 'ELSE'      # natra muji  
PROGRAM_START    = 'PROGRAM_START'  # oe muji  
PROGRAM_END      = 'PROGRAM_END'    # lala muji  
LBRACE           = 'LBRACE'  
RBRACE           = 'RBRACE'  
EQ               = 'EQ'        # ==  
NE               = 'NE'        # !=  
LT               = 'LT'        # <  
GT               = 'GT'        # >  
LTE              = 'LTE'       # <=  
GTE              = 'GTE'       # >=  
STRING           = 'STRING'  
FOR              = 'FOR'  
WHILE            = 'WHILE'  


# lexer.py  
  
import re  
from token_types import *  
  
class Token:  
    def __init__(self, type_, value):  
        self.type = type_  
        self.value = value  
    def __repr__(self):  
        return f'Token({self.type}, {repr(self.value)})'  
  
class Lexer:  
    def __init__(self, text):  
        self.text = text  
        self.pos = 0  
        self.current_char = self.text[self.pos] if self.text else None  
    def advance(self):  
        self.pos += 1  
        if self.pos >= len(self.text):  
            self.current_char = None  
        else:  
            self.current_char = self.text[self.pos]  
    def skip_whitespace(self):  
        while self.current_char is not None and self.current_char.isspace():  
            self.advance()  
    def peek(self, length):  
        peek_pos = self.pos  
        chars = ''  
        for _ in range(length):  
            if peek_pos >= len(self.text):  
                break  
            chars += self.text[peek_pos]  
            peek_pos += 1  
        return chars  
    def match_keyword(self, keywords):  
        for keyword, token_type in keywords.items():  
            length = len(keyword)  
            if self.peek(length) == keyword:  
                for _ in range(length):  
                    self.advance()  
                return Token(token_type, keyword)  
        return None
    def number(self):  
        result = ''  
        has_dot = False  
        while self.current_char is not None and (self.current_char.isdigit() or self.current_char == '.'):  
            if self.current_char == '.':  
                if has_dot:  
                    break  
                has_dot = True  
            result += self.current_char  
            self.advance()  
        if has_dot:  
            return Token(FLOAT, float(result))  
        else:  
            return Token(INTEGER, int(result))  
    def identifier(self):  
        result = ''  
        while self.current_char is not None and (self.current_char.isalnum() or self.current_char == '_'):  
            result += self.current_char  
            self.advance()  
        return Token(IDENTIFIER, result)  
    
    def string(self):  
        result = ''  
        self.advance()  # Skip the opening quote  
        while self.current_char is not None and self.current_char != '"':  
            result += self.current_char  
            self.advance()  
        if self.current_char == '"':  
            self.advance()  # Skip the closing quote  
            return Token(STRING, result)  
        else:  
            raise Exception('Unterminated string literal')
        
    def get_next_token(self):  
        while self.current_char is not None:  
            if self.current_char.isspace():  
                self.skip_whitespace()  
                continue  
            # Match multi-word keywords  
            keywords = {  
            'oe muji': PROGRAM_START,  
            'lala muji': PROGRAM_END,  
            'van muji': INPUT,  
            'her muji': PRINT,  
            'dey muji': VAR_DECL,  
            'yedi muji': IF,  
            'natra bhane muji': ELIF,  # Now ELIF  
            'natra muji': ELSE,        # Now ELSE  
            'gardai gar muji': FOR,  
            'jaba samma muji': WHILE  
        } 
            token = self.match_keyword(keywords)  
            if token:  
                return token  
            if self.current_char == '"':  
                return self.string()
            if self.current_char.isdigit():  
                return self.number()  
            if self.current_char.isalpha() or self.current_char == '_':  
                return self.identifier()  
            if self.current_char == '+':  
                self.advance()  
                return Token(PLUS, '+')  
            if self.current_char == '-':  
                self.advance()  
                return Token(MINUS, '-')  
            if self.current_char == '*':  
                self.advance()  
                return Token(MUL, '*')  
            if self.current_char == '/':  
                self.advance()  
                return Token(DIV, '/')  
            if self.current_char == '(':  
                self.advance()  
                return Token(LPAREN, '(')  
            if self.current_char == ')':  
                self.advance()  
                return Token(RPAREN, ')')  
            if self.current_char == '{':  
                self.advance()  
                return Token(LBRACE, '{')  
            if self.current_char == '}':  
                self.advance()  
                return Token(RBRACE, '}')  
            if self.current_char == ';':  
                self.advance()  
                return Token(SEMI, ';')  
            if self.current_char == '=':  
                self.advance()  
                if self.current_char == '=':  
                    self.advance()  
                    return Token(EQ, '==')  
                else:  
                    return Token(ASSIGN, '=')  
            if self.current_char == '!':  
                self.advance()  
                if self.current_char == '=':  
                    self.advance()  
                    return Token(NE, '!=')  
                else:  
                    raise Exception('Invalid character: !')  
            if self.current_char == '<':  
                self.advance()  
                if self.current_char == '=':  
                    self.advance()  
                    return Token(LTE, '<=')  
                else:  
                    return Token(LT, '<')  
            if self.current_char == '>':  
                self.advance()  
                if self.current_char == '=':  
                    self.advance()  
                    return Token(GTE, '>=')  
                else:  
                    return Token(GT, '>')  
            raise Exception(f'Invalid character: {self.current_char}')  
        return Token(EOF, None)  


# interpreter.py  
  
from parser import Parser  
from token_types import *  
  
class NodeVisitor:  
    def visit(self, node):  
        method_name = 'visit_' + type(node).__name__  
        visitor = getattr(self, method_name, self.generic_visit)  
        return visitor(node)  
    def generic_visit(self, node):  
        raise Exception('No visit_{} method'.format(type(node).__name__))  
  
class Interpreter(NodeVisitor):  
    def __init__(self, parser):  
        self.parser = parser  
        self.GLOBAL_MEMORY = {}  

    def visit_String(self, node):  
        return node.value
    
    def visit_BinOp(self, node):  
        left = self.visit(node.left)  
        right = self.visit(node.right)  
        if node.op.type == PLUS:  
            if isinstance(left, str) or isinstance(right, str):  
                return str(left) + str(right)  
            else:  
                return left + right   
        elif node.op.type == MINUS:  
            return left - right  
        elif node.op.type == MUL:  
            return left * right  
        elif node.op.type == DIV:  
            return left / right  
        elif node.op.type in (EQ, NE, LT, GT, LTE, GTE):  
            if node.op.type == EQ:  
                return left == right  
            elif node.op.type == NE:  
                return left != right  
            elif node.op.type == LT:  
                return left < right  
            elif node.op.type == GT:  
                return left > right  
            elif node.op.type == LTE:  
                return left <= right  
            elif node.op.type == GTE:  
                return left >= right  
        else:  
            raise Exception('Unsupported operator')  
          
    def visit_Num(self, node):  
        return node.value  
    def visit_Var(self, node):  
        var_name = node.name  
        value = self.GLOBAL_MEMORY.get(var_name)  
        if value is None:  
            raise Exception(f'Name "{var_name}" is not defined')  
        else:  
            return value  
    def visit_VarDecl(self, node):  
        var_name = node.var_node.name  
        if var_name in self.GLOBAL_MEMORY:  
            raise Exception(f'Variable "{var_name}" is already declared')  
        if node.expr_node is not None:  
            self.GLOBAL_MEMORY[var_name] = self.visit(node.expr_node)  
        else:  
            self.GLOBAL_MEMORY[var_name] = None  # Initialize to None  

    def visit_Assign(self, node):  
        var_name = node.left.name  
        if var_name not in self.GLOBAL_MEMORY:  
            raise Exception(f'Variable "{var_name}" is not declared')  
        self.GLOBAL_MEMORY[var_name] = self.visit(node.right)  
    def visit_Compound(self, node):  
        for child in node.children:  
            self.visit(child)  
    def visit_Print(self, node):  
        value = self.visit(node.expr)  
        print(value)  
    def visit_Input(self, node):  
        var_name = node.var_node.name  
        if var_name not in self.GLOBAL_MEMORY:  
            raise Exception(f'Variable "{var_name}" is not declared')  
        user_input = input()  
        try:  
            value = int(user_input)  
        except ValueError:  
            try:  
                value = float(user_input)  
            except ValueError:  
                value = user_input  # Keep as string if not a number  
        self.GLOBAL_MEMORY[var_name] = value  
    def visit_IfStatement(self, node):  
        executed = False  
        for condition, block in node.cases:  
            if self.visit(condition):  
                self.visit(block)  
                executed = True  
                break  
        if not executed and node.else_block:  
            self.visit(node.else_block)  

    def visit_ForLoop(self, node):  
        # Initialize the loop variable  
        self.visit(node.init)  
        while self.visit(node.condition):  
            # Execute the loop body  
            self.visit(node.body)  
            # Execute the increment statement  
            self.visit(node.increment)  
  
    def visit_WhileLoop(self, node):  
        while self.visit(node.condition):  
            self.visit(node.body)

    def interpret(self):  
        tree = self.parser.parse()  
        self.visit(tree)  


# parser.py  
  
from lexer import Token, Lexer  
from token_types import *  
  
class AST:  
    pass  
  
class BinOp(AST):  
    def __init__(self, left, op, right):  
        self.left = left  
        self.op = op  
        self.right = right  
  
class Num(AST):  
    def __init__(self, token):  
        self.token = token  
        self.value = token.value  
  
class Var(AST):  
    def __init__(self, token):  
        self.token = token  
        self.name = token.value  
  
class Assign(AST):  
    def __init__(self, left, op, right):  
        self.left = left  # Var  
        self.op = op      # Token(ASSIGN)  
        self.right = right  
  
class VarDecl(AST):  
    def __init__(self, var_node, expr_node):  
        self.var_node = var_node  
        self.expr_node = expr_node  
  
class Compound(AST):  
    def __init__(self):  
        self.children = []  
  
class Print(AST):  
    def __init__(self, expr):  
        self.expr = expr  
  
class Input(AST):  
    def __init__(self, var_node):  
        self.var_node = var_node  
  
class IfStatement(AST):  
    def __init__(self):  
        self.cases = []  # List of tuples (condition, block)  
        self.else_block = None  

class String(AST):  
    def __init__(self, token):  
        self.token = token  
        self.value = token.value  

class ForLoop(AST):  
    def __init__(self, init, condition, increment, body):  
        self.init = init          # Initialization statement  
        self.condition = condition  
        self.increment = increment  
        self.body = body  
  
class WhileLoop(AST):  
    def __init__(self, condition, body):  
        self.condition = condition  
        self.body = body 
  
class Parser:  
    def __init__(self, lexer):  
        self.lexer = lexer  
        self.current_token = self.lexer.get_next_token()  
        self.peek_token = self.lexer.get_next_token()  
    def error(self):  
        raise Exception('Invalid syntax')  
    def eat(self, token_type):  
        if self.current_token.type == token_type:  
            self.current_token = self.peek_token  
            self.peek_token = self.lexer.get_next_token()  
        else:  
            self.error()  
    def factor(self):  
        token = self.current_token  
        if token.type in (PLUS, MINUS):  
            self.eat(token.type)  
            node = self.factor()  
            return BinOp(left=Num(Token(INTEGER, 0)), op=token, right=node)  
        elif token.type in (INTEGER, FLOAT):  
            self.eat(token.type)  
            return Num(token)  
        elif token.type == IDENTIFIER:  
            self.eat(IDENTIFIER)  
            return Var(token)  
        elif token.type == STRING:  
            self.eat(STRING)  
            return String(token)  
        elif token.type == LPAREN:  
            self.eat(LPAREN)  
            node = self.expr()  
            self.eat(RPAREN)  
            return node  
        else:  
            self.error()   
    def term(self):  
        node = self.factor()  
        while self.current_token.type in (MUL, DIV):  
            token = self.current_token  
            self.eat(token.type)  
            node = BinOp(left=node, op=token, right=self.factor())  
        return node  
    def expr(self):  
        node = self.term()  
        while self.current_token.type in (PLUS, MINUS):  
            token = self.current_token  
            self.eat(token.type)  
            node = BinOp(left=node, op=token, right=self.term())  
        return node  
    def comparison(self):  
        node = self.expr()  
        while self.current_token.type in (EQ, NE, LT, GT, LTE, GTE):  
            token = self.current_token  
            self.eat(token.type)  
            node = BinOp(left=node, op=token, right=self.expr())  
        return node  
    def variable_declaration(self):  
        self.eat(VAR_DECL)  # dey muji  
        var_node = Var(self.current_token)  
        self.eat(IDENTIFIER)  
        expr_node = None  
        if self.current_token.type == ASSIGN:  
            self.eat(ASSIGN)  
            expr_node = self.expr()  
        self.eat(SEMI)  
        node = VarDecl(var_node, expr_node)  
        return node  
    def assignment(self):  
        var_node = Var(self.current_token)  
        self.eat(IDENTIFIER)  
        self.eat(ASSIGN)  
        expr_node = self.expr()  
        self.eat(SEMI)  
        node = Assign(var_node, Token(ASSIGN, '='), expr_node)  
        return node  
    def print_statement(self):  
        self.eat(PRINT)  # her muji  
        expr = self.expr()  
        self.eat(SEMI)  
        return Print(expr)  
    def input_statement(self):  
        self.eat(INPUT)  # van muji  
        var_node = Var(self.current_token)  
        self.eat(IDENTIFIER)  
        self.eat(SEMI)  
        return Input(var_node)  
    def if_statement(self):  
        node = IfStatement()  
        self.eat(IF)  # yedi muji  
        condition = self.comparison()  
        self.eat(LBRACE)  
        block = self.statement_list()  
        self.eat(RBRACE)  
        node.cases.append((condition, block))  
        while self.current_token.type == ELIF:  
            self.eat(ELIF)  # natra bhane muji now corresponds to ELIF  
            condition = self.comparison()  
            self.eat(LBRACE)  
            block = self.statement_list()  
            self.eat(RBRACE)  
            node.cases.append((condition, block))  
        if self.current_token.type == ELSE:  
            self.eat(ELSE)  # natra muji now corresponds to ELSE  
            self.eat(LBRACE)  
            node.else_block = self.statement_list()  
            self.eat(RBRACE)  
        return node  
    
    def for_loop(self):  
        self.eat(FOR)  # gardai gar muji  
        self.eat(LPAREN)  
        init = self.statement()  
        condition = self.comparison()  
        self.eat(SEMI)  
        increment = self.statement()  
        self.eat(RPAREN)  
        self.eat(LBRACE)  
        body = self.statement_list()  
        self.eat(RBRACE)  
        node = ForLoop(init, condition, increment, body)  
        return node  
  
    def while_loop(self):  
        self.eat(WHILE)  # jaba samma muji  
        condition = self.comparison()  
        self.eat(LBRACE)  
        body = self.statement_list()  
        self.eat(RBRACE)  
        node = WhileLoop(condition, body)  
        return node 
    
    def statement(self):  
        if self.current_token.type == VAR_DECL:  
            node = self.variable_declaration()  
        elif self.current_token.type == IDENTIFIER and self.peek_token.type == ASSIGN:  
            node = self.assignment()  
        elif self.current_token.type == PRINT:  
            node = self.print_statement()  
        elif self.current_token.type == INPUT:  
            node = self.input_statement()  
        elif self.current_token.type == IF:  
            node = self.if_statement()  
        elif self.current_token.type == FOR:  
            node = self.for_loop()  
        elif self.current_token.type == WHILE:  
            node = self.while_loop()  
        else:  
            self.error()  
        return node  
    
    def statement_list(self):  
        nodes = []  
        while self.current_token.type != EOF and self.current_token.type != RBRACE and self.current_token.type != PROGRAM_END:  
            node = self.statement()  
            nodes.append(node)  
        compound = Compound()  
        compound.children = nodes  
        return compound  
    def program(self):  
        self.eat(PROGRAM_START)  
        node = self.statement_list()  
        self.eat(PROGRAM_END)  
        if self.current_token.type != EOF:  
            self.error()  
        return node  
    def parse(self):  
        return self.program()  


#!/usr/bin/env python3  
  
import sys  
from lexer import Lexer  
from parser import Parser  
from interpreter import Interpreter  
  
def main():  
    if len(sys.argv) != 2:  
        print("Usage: muji <filename.muji>")  
        sys.exit(1)  
  
    filename = sys.argv[1]  
    if not filename.endswith('.muji'):  
        print("Error: File must have a .muji extension")  
        sys.exit(1)  
  
    try:  
        with open(filename, 'r') as file:  
            text = file.read()  
    except FileNotFoundError:  
        print(f"Error: File '{filename}' not found")  
        sys.exit(1)  
  
    lexer = Lexer(text)  
    parser = Parser(lexer)  
    interpreter = Interpreter(parser)  
  
    try:  
        interpreter.interpret()  
    except Exception as e:  
        print(e)  
  
if __name__ == '__main__':  
    main()  


from setuptools import setup, find_packages  
  
setup(  
    name='muji-lang',  
    version='1.0.0',  
    packages=find_packages(),  
    entry_points={  
        'console_scripts': [  
            'muji=muji.main:main',  
        ],  
    },  
    author='Your Name',  
    author_email='guluguluman4@gmail.com',  
    description='Muji Programming Language Interpreter',  
    # long_description=open('README.md').read(),  
    # long_description_content_type='text/markdown',  
    url='https://github.com/theSandeshStha/muji-bhasa',  
    classifiers=[  
        'Programming Language :: Python :: 3',  
        'Operating System :: OS Independent',  
    ],  
    python_requires='>=3.6',  
)  